@@ -57,18 +57,17 @@ GHashTable *capplet_hash = NULL;
 static char * 
 find_icon (const char *icon, GnomeDesktopItem *dentry) 
 {
-        char *icon_file = strdup (icon);
+        char *icon_file = NULL;
 
-	if (icon_file[0] == 0) {
-		g_free(icon_file);
-		icon_file = NULL;
+	if (icon && icon[0]) {
+		icon_file = g_strdup (icon);
 	}
 	
 	if (icon_file) {
 		if (icon_file[0] != '/')
 		{
 			gchar *old = icon_file;
-			icon_file = g_build_filename (PIXMAPS_DIR, old, NULL);
+			icon_file = g_build_filename (GNOMECC_ICONS_DIR, old, NULL);
 			g_free (old);
 		}
 		if (!g_file_test (icon_file, G_FILE_TEST_EXISTS) || g_file_test(icon_file, G_FILE_TEST_IS_DIR))
@@ -206,7 +205,7 @@ capplet_dir_new (CappletDir *dir, gchar *dir_path)
 
 	entry->dir = dir;
 
-	g_hash_table_insert (capplet_hash, dir_path, entry);
+	g_hash_table_insert (capplet_hash, g_strdup (dir_path), entry);
 
 	capplet_dir_load (CAPPLET_DIR (entry));
 
